/*= OX860 Metrix ========================================================= */
/* LabWindows Instrument Driver for OX860.                                 */
/* Original Release: November/29/95                                        */
/* By:  H. CHOL & F. VALOTEAU                                              */
/* Originally written in C                                                 */
/* Modification History:                                                   */
/*=========================================================================*/
#include "formatio.h"
#include "rs232.h"
#include "ox860.h"


/*= STATIC VARIABLES ======================================================*/
/* port contains the number of the port opened for the instrument module.  */
/* vert_mode contains the current vertical mode of the instrument.         */
/* ox860_err: the error variable for the instrument module                 */
/*=========================================================================*/
static short port;
static short vert_mode;
static short ox860_err;


/*= UTILITY ROUTINES ======================================================*/
int ox860_invalid_short_range (short val, short min, short max, int err_code);
int ox860_invalid_integer_range (int val, int min, int max, int err_code);
int ox860_invalid_longint_range (long val, long min, long max, int err_code);
int ox860_invalid_real_range (double val, double min, double max, int err_code);
int ox860_device_closed (void);
int ox860_send_command (int, int);

/*=========================================================================*/
/* This function opens a com port for the instrument module, queries for   */
/* ID, and initializes the instrument to a known state.                    */
/*=========================================================================*/
int ox860_init (int rs_port)
{
    char identifier[3];

    ox860_err = 0;
    if (ox860_invalid_integer_range (rs_port,1,8,-1) != 0){
        return ox860_err;
    }

    /*   Open And Configure RS232  */
    OpenComConfig (rs_port, "", 9600, 0, 8, 1, 0, 0);
    if (rs232err != 0) {
        ox860_err = rs232err+300;
        return ox860_err;
    }
    /*   Time-out : 1 Sec   */
    SetComTime (rs_port, 1.0);
    if (rs232err != 0) {
        ox860_err = rs232err+300;
        return ox860_err;
    }
    /*   Sets XON/XOFF ON   */
    SetXMode (rs_port, 1);
    if (rs232err != 0) {
        ox860_err = rs232err+300;
        return ox860_err;
    }
    port = rs_port;

    /*   Identify instrument   */
    if (ox860_send_command (OX860_ASK_ECHO, 0) !=0 ){
        ox860_close();
        return ox860_err;
    }
    ComRdTerm (port, identifier, 03, 04);
    if (identifier[1]!=0x3a){
         ox860_close();
         return 223;
    }

    /*   Sets the Instrument with default values  */
    /*    Beam Finder   :   OFF             */
    /*    Vertical mode :   ALT             */
    /*    BWL           :   OFF				*/
    /*    CH1           : AC    0.1V/Div    */
    /*    Inversion CH2 :   OFF             */
    /*    CH2           : AC    0.1V/Div    */
    /*    Time base     : x10 OFF,TBA=50S/Div,DELAY OFF,B TRIG OFF,TBB=20S/DIV,1 STEP */
    /*    Trigger       : PTP ON,SLOPE +,AUTO ON,SOURCE CH1,COUPLING DC  */
    vert_mode = 2; /* ALT */
    if (ox860_display (2,0,0) != 0){
        return ox860_err;
    }
    if (ox860_channel (0, 0, 5,0) != 0){
        return ox860_err;
    }
    if (ox860_channel (1, 0, 5,0) !=0){
        return ox860_err;
    }
    if (ox860_timebase (0, 10, 0, 0, 11, 0) != 0){
        return ox860_err;
    }
    if (ox860_trigger (1, 0, 1, 0, 0) != 0){
        return ox860_err;
    }
    return ox860_err;
}
/*---------------------------------------------------------------------*/
/*                 Instrument Setup subroutine                         */
/*---------------------------------------------------------------------*/
int ox860_setup (int rw_config, char *file)
{
    int     file_id;
    short   i,tab_fonct[19];
    char    buff[20];

    tab_fonct[0] = OX860_WRI_CONF;
    tab_fonct[1] = OX860_VERTICAL_MODE;
    tab_fonct[2] = OX860_COUPLING_CH1;
    tab_fonct[3] = OX860_RANGE_CH2;
    tab_fonct[4] = OX860_COUPLING_CH2;
    tab_fonct[5] = OX860_INVERT_CH2;
    tab_fonct[6] = OX860_TB_DELAY;
    tab_fonct[7] = OX860_RANGE_TIME_BASE_A;
    tab_fonct[8] = OX860_RANGE_TIME_BASE_B;
    tab_fonct[9] = OX860_TRIGGER_B;
    tab_fonct[10] = OX860_EXP10;
    tab_fonct[11] = OX860_TRIGGER_AUTO;
    tab_fonct[12] = OX860_TRACE_SEP;
    tab_fonct[13] = OX860_TRIGGER_COUPLING;
    tab_fonct[14] = OX860_TRIGGER_SOURCE;
    tab_fonct[15] = OX860_TRIGGER_PTP;
    tab_fonct[16] = OX860_TRIGGER_SLOPE;
    tab_fonct[17] = OX860_BWL;
    tab_fonct[18] = OX860_RANGE_CH1;

    ox860_err = 0;
    if (ox860_device_closed() != 0 ){
        return ox860_err;
    }
    if (ox860_invalid_integer_range (rw_config,0,1,-1) != 0){
        return ox860_err;
    }
    if (!rw_config)  {
        /* Save */
        file_id = OpenFile (file, 2, 0, 1);
        if (file_id ==  -1)  {
            ox860_err =  -2;
            return ox860_err;
        }
        /* Input Buffer Empty */
        FlushInQ (port);
        if (rs232err != 0) {
            ox860_err = rs232err+300;
            return ox860_err;
        }
        if (ox860_send_command (OX860_READ_SETUP, 0) != 0 ){
            return ox860_err;
        }
        i = 0;
        while (i < 19)  {
            ComRdByte (port);
            if (rs232err != 0)  {
                ox860_err = rs232err+300;
                return ox860_err;
            }
            buff[i] = ComRdByte (port);
            if (rs232err != 0)  {
                ox860_err = rs232err+300;
                return ox860_err;
            }
            ComRdByte (port);
            if (rs232err != 0)  {
                ox860_err = rs232err+300;
                return ox860_err;
            }
            i++;
        }
        /* Invert BWL and DELAY */
        buff[19] = buff[17];
        buff[17] = buff[6];
        buff[6] = buff[19];
        buff[19] = 0;
        /* Invert VERTICAL MODE and RANGE CH1 */
        buff[19] = buff[18];
       	buff[18] = buff[1];
       	buff[1] = buff[19];
       	buff[19] = 0;

        if (WriteFile (file_id, buff, 19) == -1){
            ox860_err =  230;
            return ox860_err;
        }
        if (CloseFile (file_id) == -1){
            ox860_err = -2 ;
            return ox860_err;
        }
    }
    else  {
        /* Recall */
        file_id = OpenFile (file, 1, 0, 1);
        if (file_id ==  -1)  {
            ox860_err =  -2;
            return ox860_err;
        }
        if (ReadFile (file_id, buff, 19) ==  -1)  {
            ox860_err =  231;
            return ox860_err;
        }
        i = 0;
        vert_mode = buff[1] - 0x30;
        while (i < 19)  {
            if (ox860_send_command (tab_fonct[i], buff[i] - 0x30) != 0 ){
                return ox860_err;
            }
            i++;
        }
        if (CloseFile (file_id) == -1 ){
            ox860_err =  -2;
            return ox860_err;
        }

    }
    return ox860_err;
}
/*---------------------------------------------------------------------*/
/*                  Autoset subroutine                                 */
/*---------------------------------------------------------------------*/
int ox860_autoset (void)
{
    ox860_err = 0;
    if (ox860_device_closed() != 0 ){
        return ox860_err ;
    }

    if (ox860_send_command(OX860_COM_AUTOSET,0) !=0 ){
        return ox860_err;
    }
    return ox860_err;
}
/*---------------------------------------------------------------------*/
/*                  Display settings subroutine                        */
/*---------------------------------------------------------------------*/
int ox860_display (int mod,int bwl,int beam)
{
    ox860_err = 0;
    if (ox860_device_closed() != 0 ){
        return ox860_err ;
    }
    if (ox860_invalid_integer_range (mod,0,5,-1) != 0){
        return ox860_err;
    }
    if (ox860_invalid_integer_range (bwl,0,1,-2) != 0){
        return ox860_err;
    }
    if (ox860_invalid_integer_range (beam,0,1,-3) != 0){
        return ox860_err;
    }
    if (ox860_send_command (OX860_VERTICAL_MODE, mod) != 0){
        return ox860_err;
    }
    vert_mode = mod;
    /* Bwl */
    if (ox860_send_command (OX860_BWL, bwl) != 0){
        return ox860_err;
    }
    /* Beam Finder */
    if (ox860_send_command (OX860_BEAM_FINDER, beam) != 0){
        return ox860_err;
    }
    return ox860_err;
}
/*---------------------------------------------------------------------*/
/*                 Channel Settings subroutine                         */
/*---------------------------------------------------------------------*/
int ox860_channel (int channel_num, int coupling, int sensibility,int inversion)
{
    ox860_err = 0;
    if (ox860_device_closed() != 0 ){
        return ox860_err ;
    }
    if (ox860_invalid_integer_range (channel_num,0,1,-1) != 0){
        return ox860_err;
    }
    if (ox860_invalid_integer_range (coupling,0,2,-2) != 0){
        return ox860_err;
    }
    if (ox860_invalid_integer_range (sensibility,0,10,-3) != 0){
        return ox860_err;
    }
    if (ox860_invalid_integer_range (inversion,0,1,-4) != 0){
        return ox860_err;
    }
    if (!channel_num){
        if (ox860_send_command (OX860_COUPLING_CH1, coupling) != 0)
            return ox860_err;
    }
    else{
        if (ox860_send_command (OX860_COUPLING_CH2, coupling) != 0)
            return ox860_err;
    }
    if (!channel_num){
        if (ox860_send_command (OX860_RANGE_CH1, sensibility) != 0)
            return ox860_err;
    }
    else{
        if (ox860_send_command (OX860_RANGE_CH2, sensibility) != 0){
            return ox860_err;
        }
    }
    /* Channel invert */
    if (ox860_send_command (OX860_INVERT_CH2, inversion) !=0){
        return ox860_err;
    }

    return ox860_err;
}
/*---------------------------------------------------------------------*/
/*                     Time base subroutine                            */
/*---------------------------------------------------------------------*/
int ox860_timebase (int expend,int range_A,int dlay,int B_trig,int range_B,int nb_step)
{

    ox860_err = 0;
    if (ox860_device_closed() != 0 ){
        return ox860_err ;
    }
    if (ox860_invalid_integer_range (expend,0,1,-1) != 0){
        return ox860_err;
    }
    if (ox860_invalid_integer_range (range_A,0,19,-2) != 0){
        return ox860_err;
    }
    if (ox860_invalid_integer_range (dlay,0,2,-3) != 0){
        return ox860_err;
    }
    if (ox860_invalid_integer_range (B_trig,0,1,-4) != 0){
        return ox860_err;
    }
    if (ox860_invalid_integer_range (range_B,0,19,-5) != 0){
        return ox860_err;
    }
    if (ox860_invalid_integer_range (nb_step,0,15,-6) != 0){
        return ox860_err;
    }
    /* if XY mode, error -7 */
    if (vert_mode == 5)  {
        ox860_err =  -7;
        return ox860_err ;
    }
    /* if mode delay or alterned */
    if (dlay != 0) {
        /* if Range B <= Range A, error -8 */
        if (range_B < range_A) {
            ox860_err = -8;
            return ox860_err;
        }
        if (ox860_send_command (OX860_RANGE_TIME_BASE_B, range_B) != 0){
            return ox860_err;
        }
        if (ox860_send_command (OX860_TRIGGER_B, B_trig) != 0){
             return ox860_err;
        }
        if (ox860_send_command (OX860_TRACE_SEP, nb_step) != 0){
             return ox860_err;
        }
    }
    if (ox860_send_command (OX860_EXP10, expend) != 0){
        return ox860_err;
    }
    if (ox860_send_command (OX860_RANGE_TIME_BASE_A, range_A) != 0){
        return ox860_err;
    }
    if (ox860_send_command (OX860_TB_DELAY, dlay) != 0){
        return ox860_err;
    }
    return ox860_err;
}
/*---------------------------------------------------------------------*/
/*                     Trigger subroutine                              */
/*---------------------------------------------------------------------*/
int ox860_trigger (int ptp,int slope,int mod_auto,int source,int coupling)
{
    ox860_err = 0;
    if (ox860_device_closed() != 0 ){
        return ox860_err ;
    }
    if (ox860_invalid_integer_range (ptp,0,1,-1) != 0){
        return ox860_err;
    }
    if (ox860_invalid_integer_range (slope,0,1,-2) != 0){
        return ox860_err;
    }
    if (ox860_invalid_integer_range (mod_auto,0,1,-3) != 0){
        return ox860_err;
    }
    if (ox860_invalid_integer_range (source,0,4,-4) != 0){
        return ox860_err;
    }
    if (ox860_invalid_integer_range (coupling,0,5,-5) != 0){
        return ox860_err;
    }
    /* if XY mode, error= -6 */
    if (vert_mode == 5)  {
        ox860_err =  -6;
        return ox860_err ;
    }
    if (ox860_send_command (OX860_TRIGGER_SOURCE, source) != 0){
        return ox860_err;
    }
    if (source != 3){
        if (ox860_send_command (OX860_TRIGGER_COUPLING, coupling) != 0){
            return ox860_err;
        }
    }
    /* if SOURCE LINE, COUPLING OFF */
    else {
        if (ox860_send_command (OX860_TRIGGER_COUPLING, 6) != 0){
            return ox860_err;
        }
    }
    if (ox860_send_command (OX860_TRIGGER_PTP, ptp) != 0){
        return ox860_err;
    }
    if (ox860_send_command (OX860_TRIGGER_AUTO, mod_auto) != 0){
        return ox860_err;
    }
    if (ox860_send_command (OX860_TRIGGER_SLOPE, slope) != 0){
        return ox860_err;
    }
    return ox860_err;
}
/*----------------------------------------------------------------------*/
/* This function closes the port for the instrument module and sets the */
/* port to zero.                                                        */
/*----------------------------------------------------------------------*/
int ox860_close (void)
{
    /*  Check for device closed */
    if (ox860_device_closed() !=0){
        return ox860_err;
    }

    /* Wait for Empty output buffer */
    while ( GetOutQLen(port) != 0) {};
    /* Close the com port.  If error, set ox860_err = rs232err+300.*/
    CloseCom(port);
    if (rs232err != 0) {
        ox860_err = rs232err+300;
        return ox860_err;
    }
    port = 0;
    return ox860_err;
}
/* = UTILITY ROUTINES =====================================================*/
/*=========================================================================*/
/* Function: Invalid Short Range                                           */
/* Purpose:  This function checks a short to see if it lies between a      */
/*           minimum and maximum value.  If the value is out of range, set */
/*           the global error variable to the value err_code.  If the      */
/*           value is OK, error = 0.                                       */
/*=========================================================================*/
int ox860_invalid_short_range (short val, short min, short max, int err_code)
{
  if ((val < min) || (val > max)) {
    ox860_err = err_code;
    return -1;
  }
  return 0;
}
/*=========================================================================*/
/* Function: Invalid Integer Range                                         */
/* Purpose:  This function checks an integer to see if it lies between a   */
/*           minimum and maximum value.  If the value is out of range, set */
/*           the global error variable to the value err_code.  If the      */
/*           value is OK, error = 0.                                       */
/*=========================================================================*/
int ox860_invalid_integer_range (int val, int min, int max, int err_code)
{
  if ((val < min) || (val > max)) {
    ox860_err = err_code;
    return -1;
  }
  return 0;
}
/*=========================================================================*/
/* Function: Invalid Long Integer Range                                    */
/* Purpose:  This function checks a long integer to see if it lies between */
/*           a minimum and maximum value.  If the value is out of range,   */
/*           set the global error variable to the value err_code.  If the  */
/*           value is OK, error = 0.  The return value is equal to the     */
/*           global error value.                                           */
/*=========================================================================*/
int ox860_invalid_longint_range (long val, long min, long max, int err_code)
{
    if (val < min || val > max) {
        ox860_err = err_code;
        return -1;
    }
    return 0;
}
/*=========================================================================*/
/* Function: Invalid Real Range                                            */
/* Purpose:  This function checks a real number to see if it lies between  */
/*           a minimum and maximum value.  If the value is out of range,   */
/*           set the global error variable to the value err_code.  If the  */
/*           value is OK, error = 0.                                       */
/*=========================================================================*/
int ox860_invalid_real_range (double val, double min, double max, int err_code)
{
  if ((val < min) || (val > max)) {
    ox860_err = err_code;
    return -1;
  }
  return 0;
}
/*=========================================================================*/
/* Function: Device Closed                                                 */
/* Purpose:  This function checks to see if the module has been            */
/*           initialized.  If the device has not been opened, a 1 is       */
/*           returned, 0 otherwise.                                        */
/*=========================================================================*/
int ox860_device_closed (void)
{
    if (port== 0){
        ox860_err = 232;
        return -1;
    }
    return 0 ;

}
/*=========================================================================*/
/* Function: Send Command                                                  */
/* Purpose: This function sends a command to the instrument. If the        */
/*          scope is not initialized, an error 232 is returned.            */
/*=========================================================================*/
int ox860_send_command (int function, int parameter)
{
    ox860_err = 0;
    if (port != 0)  {
        parameter += 0x30;
        if (ComWrtByte (port, function) ==0){
            ox860_err = 230;
            return ox860_err ;
        }
        if (ComWrtByte (port, parameter) == 0){
            ox860_err = 230;
            return ox860_err ;
        }
        if (ComWrtByte (port, 04) ==0){
            ox860_err = 230;
            return ox860_err ;
        }
    }
    else{
        ox860_err = 232;
    }
    return ox860_err ;
}
/*= THE END ===============================================================*/
